import numpy as np
import random
import math

import matplotlib.mlab as mlab
import matplotlib.pyplot as plt
from matplotlib import animation, rc
from IPython.display import HTML

# Graph population happiness
def plot_population(fig, pop, xlabel):
    ax = fig.gca()
    counts, bins = np.histogram(pop, range(151))
    normed_counts = counts/np.sum(counts)
    ax.bar(bins[:-1], normed_counts, width=(bins[1]-bins[0]), facecolor='green', alpha=1)
    ax.set_xlabel(xlabel.capitalize())
    ax.set_ylabel('PMF')
    ax.grid(True)
    ax.set_title('Population {} (N = {})'.format(xlabel.capitalize(), len(pop)))
    plt.show(fig)

def get_pop_statistics(pop, xlabel):
    # Print population statistics
    mu = np.mean(pop)
    sigma_sq = np.var(pop)
    sigma = np.std(pop)
    print("population size, N = {}".format(len(pop)))
    print("mean %s , mu = %.2f" % (xlabel, mu))
    print("var %s, sigma^2 = %.2f" % (xlabel, sigma_sq))
    print("std deviation %s, sigma = %.2f" % (xlabel, sigma))
    return mu, sigma_sq, sigma


# Graph sample happiness
def plot_sample(fig, sample, xlabel, binwidth=1, norm=False, binmin=0, binmax=151, title=None):
    ax = fig.gca()
    bins = np.arange(binmin, binmax, step=binwidth)
    counts, bins = np.histogram(sample, bins)
    if norm:
        normed_counts = counts/np.sum(counts)
        ax.bar(bins[:-1], normed_counts, width=binwidth, facecolor='white', edgecolor='green', alpha=1)
        ax.set_ylabel('PMF',fontsize=16)
        ax.set_yticklabels([str('%.3f' % x)[1:] for x in ax.get_yticks()], fontsize=16)
#        ax.set_yticklabels([str('%.2f' % x)[1:] for x in ax.get_yticks()], fontsize=16)
    else:
        ax.bar(bins[:-1], counts, width=binwidth, facecolor='white', edgecolor='green', alpha=1)
        ax.set_ylabel('Counts',fontsize=16)
        if np.amax(counts) < 20:
            ax.set_yticks(np.arange(21, step=4))
        else:
            ax.set_yticks(np.arange(51, step=10))
       
        ax.set_yticklabels([str(int(x)) for x in ax.get_yticks()], fontsize=16)
    ax.set_xlabel(xlabel.capitalize(), fontsize=16)
    ax.set_xticklabels(map(int, ax.get_xticks()), fontsize=16)
    if title is None:
        ax.set_title('Sample {} (n = {})'.format(xlabel.capitalize(), len(sample)), fontsize=16)
        print('Sample mean', np.mean(sample))
    else:
        ax.set_title(title, fontsize=16)
    
    plt.show(fig)

def plot_sample_means(fig, means, mu, bar_x):
    #counts, bins, patches = plt.hist(means, np.arange(70, 95),facecolor='navajowhite', edgecolor='chocolate', linewidth=2,  alpha=0.75)
    counts, bins = np.histogram(means, np.arange(70, 95))
    normed_counts = counts/np.sum(counts)
    ax = fig.gca()
    
    ax.bar(bins[:-1], normed_counts, width=1, facecolor='navajowhite', edgecolor='chocolate', linewidth=2, alpha=1)
    ax.axvline(mu, color='black', label='pop mean, $\mu$')
    ax.axvline(bar_x, color='darkgreen', linestyle='dashed', label='our mean, %.2f' % bar_x)
    ax.set_xlabel('Sample mean of happiness ($n = 200$)')
    ax.set_ylabel('PMF')
    ax.legend()
    ax.set_title("Distribution of sample means")
    plt.show(fig)

def plot_bootstrapped_variances(fig, sample_variance_boots):
    counts, bins = np.histogram(sample_variance_boots, bins=20)
    normed_counts = counts/np.sum(counts)
    ax = fig.gca()
    ax.bar(bins[:-1], normed_counts, width=bins[1] - bins[0],
            facecolor='white', edgecolor='cornflowerblue', linewidth=2, alpha=1) # variances
    #ax.bar(bins[:-1], normed_counts, width=bins[1]-bins[0],
    #        facecolor='white', edgecolor='chocolate', linewidth=2, alpha=1) # means
    ax.set_title("Bootstrapped sample variances", fontsize=16)
    ax.set_xlabel('Sample variances', fontsize=16)
    ax.set_ylabel('Probability', fontsize=16)
    ax.set_yticklabels([str('%.2f' % x)[1:4] for x in ax.get_yticks()], fontsize=16)
    ax.set_xticklabels(map(int, ax.get_xticks()), fontsize=16)
    plt.show()
    
def plot_bootstrapped_differences(fig, x, num_bins, title):
    bin_width = (max(x) - min(x)) / float(num_bins)
    bins = np.arange(min(x), max(x) + bin_width, bin_width)
    n, bins, patches = plt.hist(x, bins = bins, facecolor='green', alpha=0.75)
    ax = fig.gca()
    ax.set_xlabel('Difference')
    ax.set_ylabel('Count')
    ax.grid(True)
    ax.set_title(title)
    plt.show(fig)

def printRow(row):
    s = ''
    for i in range(len(row)):
        if isNumber(row[i]):
            s += '{:.0f}'.format(float(row[i]))
        else:
            s += str(row[i])
        if i != len(row) - 1:
            s += ' '
    print(s)

def plotHistogram(x, is_sum = False, figIndex = 0, title="Data"):

    fig = plt.figure(figIndex)

    # the histogram of the data
    if not is_sum:
      n, bins, patches = plt.hist(x, range(151), facecolor='green', alpha=0.75)
    else:
      n, bins, patches = plt.hist(x, np.arange(151*15, step=15), facecolor='green', alpha=0.75)

    # add a 'best fit' line
    
    plt.xlabel('Value')
    plt.ylabel('Count')
    plt.grid(True)
    plt.title(title)

    return fig

def load(fileName):
    values = []
    with open(fileName) as f:
        for row in f:
            values.append(float(row))
    return values

def isNumber(s):
    try:
        float(s)
        return True
    except ValueError:
        return False
